@@ -55,6 +55,16 @@
 ##' Chow, S.-M., Hamaker, E. L., & Allaire, J. C. (2009).
 ##' Using innovative outliers to detect discrete shifts in dynamics in group-based state-space models. _Multivariate Behavioral Research_, 44, 465-496.
 ##' 
+##' @examples
+##' \dontrun{
+##' # See the demo for outlier detection, OutlierDetection.R
+##' dynrCook <- dynr.cook(dynrModel)
+##' dynrTaste <- dynr.taste(dynrModel, dynrCook)
+##' 
+##' # Detect outliers related to 'eta1' out of, say, three latent
+##' # variables c("eta1", "eta2", "eta3"), and all measured variables.
+##' dynrTaste <- dynr.taste(dynrModel, dynrCook, which.state=c("eta1"))
+##' }
 dynr.taste <- function(dynrModel, dynrCook=NULL,
                        which.state, which.obs,
                        conf.level=0.99,
@@ -188,7 +198,7 @@ dynr.taste <- function(dynrModel, dynrCook=NULL,
         Nnew <- t_Lambda %*% obsInfI %*% Lambda + t(Li) %*% Ni %*% Li
         N[,,i-1] <- Nnew
         Ninv_i <- try(solve(Nnew), silent=TRUE)
-        if(class(Ninv_i) == "try-error"){Ninv_i <- MASS::ginv(Nnew)}
+        if("try-error" %in% class(Ninv_i)){Ninv_i <- MASS::ginv(Nnew)}
         Ninv[,,i-1] <- Ninv_i
         chiLat[i-1] <- t(rnew) %*% Ninv_i %*% rnew
       }
@@ -278,7 +288,7 @@ dynr.taste <- function(dynrModel, dynrCook=NULL,
         Nnew <- t_Lambda %*% obsInfI %*% Lambda + t(Li) %*% Ni %*% Li
         N[,,i-1] <- Nnew
         Ninv_i <- try(solve(Nnew), silent=TRUE)
-        if(class(Ninv_i) == "try-error"){Ninv_i <- MASS::ginv(Nnew)}
+        if("try-error" %in% class(Ninv_i)){Ninv_i <- MASS::ginv(Nnew)}
         Ninv[,,i-1] <- Ninv_i
         chiLat[i-1] <- t(rnew) %*% Ninv_i %*% rnew
       }
@@ -326,7 +336,7 @@ dynr.taste <- function(dynrModel, dynrCook=NULL,
         s_j <- t(X_t) %*% u[,j, drop=FALSE] + t(W_t) %*% r[,j, drop=FALSE]
         s[,j] <- s_j##
         S_j_inv <- try(solve(S_j), silent=TRUE)
-        if(class(S_j_inv) == "try-error"){S_j_inv <- MASS::ginv(S_j)}
+        if("try-error" %in% class(S_j_inv)){S_j_inv <- MASS::ginv(S_j)}
         delta[,j] <- S_j_inv %*% s_j
         t_value[,j] <- s_j / sqrt(diag(S_j))
       }
@@ -345,7 +355,7 @@ dynr.taste <- function(dynrModel, dynrCook=NULL,
           t(Q_j) %*% matrix(N[,,j], dimLat, dimLat) %*% Q_j
         s_j <- t(X_t) %*% u[,j, drop=FALSE] + t(W_t) %*% r[,j, drop=FALSE]
         S_j_inv <- try(solve(S_j), silent=TRUE)
-        if(class(S_j_inv) == "try-error"){S_j_inv <- MASS::ginv(S_j)}
+        if("try-error" %in% class(S_j_inv)){S_j_inv <- MASS::ginv(S_j)}
         delta[,j] <- S_j_inv %*% s_j
         t_value[,j] <- s_j / sqrt(diag(S_j))
       }
@@ -496,6 +506,8 @@ dynr.taste <- function(dynrModel, dynrCook=NULL,
 ##' a new \code{dynrModel} object the outliers are applied,
 ##' and a \code{dynrCook} object the new \code{dynrModel} object is cooked.
 ##' 
+##' @examples
+##' # dynrTaste2 <- dynr.taste2(dynrModel, dynrCook, dynrTaste)
 dynr.taste2 <- function(dynrModel, dynrCook, dynrTaste,
                         delta_inn=c("t", "ind", "jnt", "null"),
                         delta_add=c("t", "ind", "jnt", "null"),
